/*
 * Copyright (C) 2010 Digitalis Education Solutions, Inc.
 * Author: Trystan Larey-Williams
 * Date: 2-22-2010
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Nightshade is a trademark of Digitalis Education Solutions, Inc.
 * See the TRADEMARKS file for trademark usage requirements.
 *
 */

/*
 * Helper class to manage named shared memory resources wraps read and write
 * calls in mutex for IPC safety
 */

#include "nshade_shared_memory.h"

using namespace boost::interprocess;
using namespace std;

NshadeSharedMemory::NshadeSharedMemory() {
	m_shm = new managed_shared_memory( open_or_create, "/nightshadeSM", 512000 );

	const ShmAlloc allocator( m_shm->get_segment_manager() );
	m_connections = m_shm->find_or_construct<ShmList>(unique_instance)(allocator);

	m_mutex = m_shm->find_or_construct<interprocess_mutex>(unique_instance)();
	m_connections = m_shm->find_or_construct<ShmList>(unique_instance)(allocator);
	m_refCount = m_shm->find_or_construct<ReferenceCounter>(unique_instance)();

	if( m_refCount->m_ref == 0 ) {
		cout << "Nightshade Shared Memory Segment Initialized.\n";
	}
	m_refCount->m_ref++;
	m_clientType = NshadeSharedMemoryConnection::user;
}

NshadeSharedMemory::~NshadeSharedMemory() {
	if( --m_refCount->m_ref == 0 ) {
		// Remove any unreleased connection objects
		for( ShmList::iterator itr = m_connections->begin(); itr != m_connections->end(); itr++ ) {
			m_shm->destroy_ptr( itr->get() );
		}
		m_connections->clear();

		// Cleanup singletons
		m_shm->destroy<NshadeCommand>(unique_instance);
		m_shm->destroy<managed_shared_memory>(unique_instance);
		m_shm->destroy<interprocess_mutex>(unique_instance);

		// Tear-down shared memory
		delete m_shm;
		shared_memory_object::remove( "nightshadeSM" );
		cout << "Nightshade Shared Memory Segment Removed.\n";
	}
	else
		delete m_shm;
}

NshadeSharedMemoryConnection* const NshadeSharedMemory::AddConnection( void ) {
	offset_ptr<NshadeSharedMemoryConnection> obj = m_shm->construct<NshadeSharedMemoryConnection>(anonymous_instance)( m_shm );
	obj->SetOwner( reinterpret_cast<const unsigned long>(this) );
	obj->SetClientType( m_clientType );
	m_mutex->lock();
	m_connections->push_back( obj );
	m_mutex->unlock();
	return obj.get();
}

void NshadeSharedMemory::SetClientType( NshadeSharedMemoryConnection::Type type ) {
	m_clientType = type;
}

bool NshadeSharedMemory::TryLock() {
	return m_mutex->try_lock();
}

void NshadeSharedMemory::Lock() {
	m_mutex->lock();
}

void NshadeSharedMemory::UnLock() {
	m_mutex->unlock();
}

void NshadeSharedMemory::RemoveConnection( NshadeSharedMemoryConnection* const connection ) {
	m_mutex->lock();
	for( ShmList::iterator itr = m_connections->begin(); itr != m_connections->end(); itr++ ) {
		if( itr->get() == connection ) {
			m_shm->destroy_ptr( itr->get() );
			m_connections->remove( *itr );
			break;
		}
	}
	m_mutex->unlock();
}

void NshadeSharedMemory::DirtyAll( void ) {
	m_mutex->lock();
	for( ShmList::iterator itr = m_connections->begin(); itr != m_connections->end(); itr++ ) {
		if( (*itr)->Owner() != reinterpret_cast<const unsigned long>(this) &&
			(*itr)->ClientType() != m_clientType ) {
				(*itr)->SetDirty();
		}
	}
	m_mutex->unlock();
}











